#  aRT : API R - TerraLib                                                
#  Copyright (C) 2003-2005  LEG                                          
#                                                                        
#  This program is free software; you can redistribute it and/or modify  
#  it under the terms of the GNU General Public License as published by  
#  the Free Software Foundation; either version 2 of the License, or     
#  (at your option) any later version.                                   
#                                                                        
#  This program is distributed in the hope that it will be useful,       
#  but WITHOUT ANY WARRANTY; without even the implied warranty of        
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
#  GNU General Public License for more details.                          
#                                                                        
#  You should have received a copy of the GNU Lesser General Public      
#  License along with this library.

aRTplot = function(geom, id=NULL, add=FALSE, ...)
{
	if(add)
	{
		if(class(geom) == "SpatialPolygons")        aRTpolygon(geom, id, ...)
		if(class(geom) == "SpatialLines")           aRTline   (geom, id, ...)
		if(class(geom) == "SpatialPointsDataFrame") aRTpoint  (geom, id, ...)
	}
	else
	{
		if(is.null(id))
		{
		    xr <- geom@bbox[1,]
		    yr <- geom@bbox[2,]
		    frame()
		    plot.window(xlim=xr, ylim=yr, asp=1)
			aRTplot(geom, add=TRUE, ...)
		}
		else if(class(geom) == "SpatialPolygons")
		{
			ids=getSpPPolygonsIDSlots(geom)
			ssrings=lapply(id, function(x) geom[which(ids ==x)]@polygons[[1]])
			SpatialPolygons(ssrings)->SR

		    xr <- SR@bbox[1,]
		    yr <- SR@bbox[2,]
		    frame()
		    plot.window(xlim=xr, ylim=yr, asp=1)

			aRTplot(SR, add=TRUE, ...)
		}
		#else if(class(geom) == "SpatialPointsDataFrame")
		#{
			
		#}
		else stop("not implemented yet")
		box()
	}
	return(invisible())
}

# transformar em apply
aRTpoint = function(geom, id=NULL, ...)
{
    if(is.null(id)) points(geom, ...)
    else
	{
		for(i in 1:(dim(geom)[1]))
		{
			if(any(id == as.vector(geom@data$ID[i])))
			{
				points(geom[i,],...)
			}
		}
	}
}

aRTpolygon = function(geom, id=NULL, ...)
{
	for(i in 1:length(geom@polygons))
	{
		if(is.null(id) || any(id == geom@polygons[[i]]@ID))
		{
			for(j in 1:length(geom@polygons[[i]]@Polygons))
			{
				polygon(geom@polygons[[i]]@Polygons[[j]]@coords, ...)
			}
		}
	}
}

aRTline = function(geom, id=NULL, ...)
{
    for(i in 1:length(geom@lines))
    {
        if(is.null(id) || any(id == geom@lines[[i]]@ID))
        {
            for(j in 1:length(geom@lines[[i]]@Lines))
            {
                lines(geom@lines[[i]]@Lines[[j]]@coords, ...)
            }
        }
    }
}


setGeneric("plot",  function(x,y, ...) standardGeneric("plot"))
setMethod("plot", "aRTtheme", function(x, y, ...) {
  out <- getGeometry(x)
  plot(out, ...)
  return(invisible())
})

setMethod("plot", "aRTlayer", function(x, y, ...) {
  summ = summary(x)
  if(summ$raster == "yes" ) {  out <- getRaster(x); image(out, asp=1, ...) }
  if(summ$polygons > 0    ) {  out <- getPolygons(x); aRTplot(out, ...)    }
  if(summ$lines > 0       ) {  out <- getLines(x);    aRTplot(out, ...)    }
  if(summ$points > 0      ) {  out <- getPoints(x);   aRTplot(out, ...)    }
  return(invisible())
})

.selectData <- function(data, id)
{
	if(!is.null(data$polygons))
	{
		data$polygons<-data$polygons[which(is.element(data$id, id))]
	}
	if(!is.null(data$points))
	{
	    data$points<-data$points[which(is.element(data$id, id))]
	}
	if(!is.null(data$lines))
	{
	    data$lines<-data$lines[which(is.element(data$id, id))]
	}
					
	data
}

plot.aRTgeometry <- function(x, save = FALSE, add = FALSE, id = NULL, ...)
# implementar para plotar apenas os ids passados como argumento
{
	if(!is.null(id))
		x<- .selectData(x, id)

	if(!add)
	{
	   	grid<- .expand( .removenulls(c(x$polygons, x$points, x$lines, list()) ))
    	plot(expand.grid(grid), type="n", asp=1, ...)
	}

	.aRTplot(x, ...)
}

.aRTplot <- function(data, ...)
{
	if( any(is.element(names(data), "polygons")) ) .plotpol( .removenulls(data$polygons), ... )
    if( any(is.element(names(data), "points"  )) ) .plotpoints( .removenulls(data$points), ... )
    if( any(is.element(names(data), "lines"   )) ) .plotlines( .removenulls(data$lines), ... )
}

# remove all positions that are null, and then cant be plotted
.removenulls <- function(data)
{
	return ( data[which(!sapply(data, is.null))] )
}

.expand <- function(data)
{
    p<-list(x=range(sapply(data, function(x) range(x$x))),
            y=range(sapply(data, function(x) range(x$y))))
	return (p)
}

.plotpol <-function(pols, ...)
{
	# draws each point repetition as one polygon
    for(i in 1:length(pols))
	{
		pol <- pols[[i]]
		j<-1

		for(k in 2:length(pol$x))
		{
		    if(j != k && pol$x[j] == pol$x[k] && pol$y[j] == pol$y[k])
		    {
				polygon( list(x=pol$x[j:k], y = pol$y[j:k]), ... )
		        j<-k+1
		    }
		}		
	}
    return(invisible())
}

# cex, pch
.plotpoints <- function(points, ...)
{
	sapply( 1:length(points), function(x)  points(points[[x]], ...))
	return(invisible())
}

.plotlines <- function(lines, ...)
{
	sapply( 1:length(lines), function(x)  lines(lines[[x]]$x, lines[[x]]$y, ...))
	return(invisible())
}
