#  aRT : API R - TerraLib                                                
#  Copyright (C) 2003-2005  LEG                                          
#                                                                        
#  This program is free software; you can redistribute it and/or modify  
#  it under the terms of the GNU General Public License as published by  
#  the Free Software Foundation; either version 2 of the License, or     
#  (at your option) any later version.                                   
#                                                                        
#  This program is distributed in the hope that it will be useful,       
#  but WITHOUT ANY WARRANTY; without even the implied warranty of        
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
#  GNU General Public License for more details.                          
#                                                                        
#  You should have received a copy of the GNU Lesser General Public      
#  License along with this library.

require(sp)

#====================================================================
# preparing spdefs object which stores class definitions for
# building objects outside R
# it is necessary to avoid gc() kill them
.aRTspdefs = list()
.aRTspdefs$polygon  = getClassDef("Polygon")
.aRTspdefs$polygons = getClassDef("Polygons")
.aRTspdefs$line     = getClassDef("Line")
.aRTspdefs$lines    = getClassDef("Lines")

.aRTinitSpDefs = function()
{
    .Call("cppInitSpDefs", .aRTspdefs)
    return(invisible())
}

as.SpatialPoints = function(data)
{
	points = data$points
	pts = NULL
    for(i in 1:length(points))
    {
		pts = rbind(pts, cbind(x=points[[i]]$x, y=points[[i]]$y))
    }
	row.names(pts)=data$id # did not work
	SP = SpatialPoints(pts)
	return(SP)
}

# if data is null use the ids from coords
as.SpatialPointsDataFrame = function(coords, data = NULL)
{
	if(is.null(data))
		SP = SpatialPointsDataFrame(as.SpatialPoints(coords), data.frame(ID=coords$id))
	else
		SP = SpatialPointsDataFrame(as.SpatialPoints(coords), data)
	return(SP)
}


as.SpatialLines = function(data)
{
	lines = data$lines
	slines = NULL
    for(i in 1:length(lines))
    {
        line = lines[[i]]
		sline = list(Line(cbind(line$x, line$y)))
		slines = c(slines, list(Lines(sline,ID=data$id[i])))
    }
	SL = SpatialLines(slines)
	return(SL)
}

as.SpatialPolygons = function(data)
{
	pols = data$polygons
	sring = NULL
    for(i in 1:length(pols))
    {
        pol = pols[[i]]
        j = 1
		spol = NULL
		
        for(k in 2:length(pol$x))
        {
            if(j != k && pol$x[j] == pol$x[k] && pol$y[j] == pol$y[k])
            {
				spol = c(spol, list(Polygon(cbind(pol$x[j:k], pol$y[j:k]))))
                j = k+1
            }
        }
		sring = c(sring, list(Polygons(spol, data$id[i])))
    }
	SR = SpatialPolygons(sring, 1:length(pols))
	return(SR)
}

# if data is null use the ids from coords
as.SpatialPolygonsDataFrame = function(coords, data = NULL)
{
	if(is.null(data))
		SP = SpatialPolygonsDataFrame(as.SpatialPolygons(coords), data.frame(ID=coords$id))
	else
		SP = SpatialPolygonsDataFrame(as.SpatialPolygons(coords), data)
	return(SP)
}

setGeneric("as.aRTgeometry", function(object) standardGeneric("as.aRTgeometry"))

setMethod("as.aRTgeometry", "SpatialPoints", function(object) {
	mat = coordinates(object)
    colnames(mat) = c("x", "y")

	result = NULL
	result$points = apply(mat, 1, as.list)
	result$id = paste(1:length(result$points))
	class(result) = "aRTgeometry"

	return (result)
})

setMethod("as.aRTgeometry", "SpatialPointsDataFrame", function(object) {
	mat = coordinates(object)
    colnames(mat) = c("x", "y")

	result = NULL
	result$points = apply(mat, 1, as.list)
	result$id = paste(object@data$ID)
	class(result) = "aRTgeometry"

	return (result)
})

setMethod("as.aRTgeometry", "SpatialLines", function(object) {
	return(invisible())
})

setMethod("as.aRTgeometry", "SpatialPolygons", function(object) {
	pols = getSpPPolygonsIDSlots(object)
	rpols = NULL
	for(i in 1:length(pols))
	{
		srings = getPolygonsIDSlot(pols[[i]])
		r = NULL
		for(j in length(srings):1)
		{
			r$x = c(getPolygonCoordsSlot(srings[[j]])[,1], r$x)
			r$y = c(getPolygonCoordsSlot(srings[[j]])[,2], r$y)
		}
		rpols = c(rpols, list(r))
	}
	result = NULL
	result$polygons = rpols
	result$id = getSpPPolygonsIDSlots(object)
	class(result) = "aRTgeometry"

	return (result)
})

