/**************************************************************************\
 *  aRT : API R - TerraLib                                                *
 *  Copyright (C) 2003-2005  LEG                                          *
 *                                                                        *
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation; either version 2 of the License, or     *
 *  (at your option) any later version.                                   *
 *                                                                        *
 *  This program is distributed in the hope that it will be useful,       *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU Lesser General Public      *
 *  License along with this library.                                      *
\**************************************************************************/

#include "SEXPutils.h"

#include <TeInitRasterDecoders.h>
#include <TeImportRaster.h>
#include <TeInitQuerierStrategy.h>
#include <TeQuerierParams.h>

#include <sstream>

extern "C" {

namespace SEXPutils {

SEXP POLYGON_DEF  = R_NilValue;
SEXP POLYGONS_DEF = R_NilValue;
SEXP LINES_DEF    = R_NilValue;
SEXP LINE_DEF     = R_NilValue;

void InitSpDefs(SEXP defs)
{
	POLYGON_DEF  = GetListElement(defs, "polygon" );
	POLYGONS_DEF = GetListElement(defs, "polygons");
	LINE_DEF     = GetListElement(defs, "line"    );
	LINES_DEF    = GetListElement(defs, "lines"   );
}

// visual data is a list which each attribute will be
// a data of the TeVisual
TeVisual GetTeVisual(SEXP visual, TeGeomRep geomrep)
{
	int style   = GET_INTEGER_ELEMENT(visual, "style"  );
	int width   = GET_INTEGER_ELEMENT(visual, "width"  );
	int cwidth  = GET_INTEGER_ELEMENT(visual, "cwidth" );
	int transp  = GET_INTEGER_ELEMENT(visual, "transp" );
	int ctransp = GET_INTEGER_ELEMENT(visual, "ctransp");
	int size    = GET_INTEGER_ELEMENT(visual, "size"   );
	SEXP color  = GetListElement     (visual, "color"  );
	SEXP ccolor = GetListElement     (visual, "ccolor" );
	
	TeVisual tvisual(geomrep);

    TeColor tcolor  = SEXPToColors(color)[0];
    TeColor tccolor = SEXPToColors(ccolor)[0];
	
	tvisual.style(style);
	tvisual.size(size);
	tvisual.width(width);
	tvisual.contourWidth(cwidth);
	tvisual.transparency(transp);
	tvisual.contourTransparency(ctransp);
	tvisual.color(tcolor); 
	tvisual.contourColor(tccolor);
	
	return tvisual;
}

// um unico espao separando os atributos e sem \n
TeProjection* GetTeProjection(string proj4)
{
	return TeGetTeProjectionFromSProj(proj4);
}
		
vector<TeColor> SEXPToColors(SEXP colors)
{
    vector<TeColor> colorVec;
    int size = LENGTH(colors)/3;
    
    for(int i = 0; i < size; i++)
    {
        TeColor cor(INTEGER(colors)[3*i],
                    INTEGER(colors)[3*i+1], 
                    INTEGER(colors)[3*i+2]);
        colorVec.push_back(cor);
    }
    return colorVec;
}

bool AddRaster(TeLayer* layer, SEXP rasterdata)
{
	TeInitRasterDecoders();
    TeRasterParams params;

    SEXP x   = GetListElement(rasterdata, "x"  );
	SEXP y   = GetListElement(rasterdata, "y"  );
    SEXP dim = GetListElement(rasterdata, "dim");
	
	TeProjection* proj = new TeNoProjection();

    params.nBands(1);
    params.projection(proj);
    params.nlines_ = INTEGER(dim)[1];
    params.ncols_ = INTEGER(dim)[0];

	params.boundingBoxLinesColumns(REAL(x)[0],
				        		   REAL(y)[0],
								   REAL(x)[1],
								   REAL(y)[1],
								   INTEGER(dim)[1],
								   INTEGER(dim)[0]);
    params.setDataType(TeDOUBLE);
    params.setPhotometric(TeRASTERKERNEL);
    params.decoderIdentifier_ = "MEM";
    params.mode_ = 'c';

	TeDecoderSEXP* decoder = new TeDecoderSEXP(rasterdata, params);
	TeRaster* raster = new TeRaster();
    raster->setDecoder(decoder);

    if(!raster->init()) return false;
    return TeImportRaster (layer,
                           raster,
                           INTEGER(dim)[0],
                           INTEGER(dim)[1],
                           TeNoCompression,
                           "", // object id
						   raster->params().dummy_[0],
						   true); // usedummy
}

SEXP GetBoxTeBox(TeBox box){
    SEXP *coords, colnames;
	SEXP result;

    PROTECT(result   = allocVector(VECSXP, 2));
    PROTECT(colnames = allocVector(STRSXP, 2));

    coords = new SEXP[2];

    PROTECT(coords[0] = allocVector(REALSXP,5));
    PROTECT(coords[1] = allocVector(REALSXP,5));

    SET_STRING_ELT( colnames, 0, mkChar("x") );
    SET_STRING_ELT( colnames, 1, mkChar("y") );

    REAL(coords[0])[0] = box.x1();
    REAL(coords[1])[0] = box.y1();

    REAL(coords[0])[1] = box.x2();
    REAL(coords[1])[1] = box.y1();

    REAL(coords[0])[2] = box.x2();
    REAL(coords[1])[2] = box.y2();

    REAL(coords[0])[3] = box.x1();
    REAL(coords[1])[3] = box.y2();

	REAL(coords[0])[4] = box.x1();
    REAL(coords[1])[4] = box.y1();

    SET_VECTOR_ELT(result, 0, coords[0]);
    SET_VECTOR_ELT(result, 1, coords[1]);

    setAttrib(result, R_NamesSymbol, colnames);
    delete[] coords;

    UNPROTECT(4);

    return result;
}              

SEXP GetPointsUtils(TePointSet& ps)
{
  int nroptos = ps.size() ;
  SEXP coords, rownames, colnames, dimnames;

  PROTECT(coords   = allocMatrix(REALSXP, 2, nroptos));
  PROTECT(rownames = allocVector(INTSXP, nroptos));
  PROTECT(dimnames = allocVector(VECSXP, 2));
  PROTECT(colnames = allocVector(STRSXP, 2));

  SET_STRING_ELT(colnames, 0, mkChar("x"));
  SET_STRING_ELT(colnames, 1, mkChar("y"));

  for (int j = 0; j < nroptos; j++)
  {
    REAL(coords)[0 + 2 * j] = ps[j].location().x();
    REAL(coords)[1 + 2 * j] = ps[j].location().y();

    const char *id = ps[j].objectId().c_str();

    // creates a vector with values rownames
    INTEGER(rownames)[j] = atoi(id);
  }
    
  SET_VECTOR_ELT(dimnames, 0, colnames);
  SET_VECTOR_ELT(dimnames, 1, rownames);

  setAttrib(coords, R_DimNamesSymbol, dimnames);
  UNPROTECT(4);
  return coords;
}

SEXP GetLayerData(TeLayer* layer)
{
	return R_NilValue;
}

SEXP GetThemeData(TeTheme* theme)
{
	return R_NilValue;
}

SEXP GetLayerAttributes(TeLayer* layer)
{
    TeInitQuerierStrategies(); // never forget it!

    TeQuerierParams params; // params default is no geometry, only attributes
    params.setParams(layer);
    TeQuerier* querier = new TeQuerier(params);
	SEXP result = GetQuerierAttributes(querier);
	delete querier;
	return result;
}

SEXP GetThemeAttributes(TeTheme* theme)
{
    TeInitQuerierStrategies(); // never forget it!

    TeQuerierParams params; // params default is no geometry, only attributes
    params.setParams(theme);
    TeQuerier* querier = new TeQuerier(params);
	SEXP result = GetQuerierAttributes(querier);
	delete querier;
	return result;
}

SEXP GetQuerierAttributes(TeQuerier* querier)
{
	SEXP colnames, result;
	SEXP* each_column;
	TeSTInstance sti;
	TePropertyVector pv;
	int size = 0;
	int lines = 0;
	
	if( !querier->loadInstances() )    return R_NilValue;
	if( !querier->fetchInstance(sti) ) return R_NilValue;

	// first time we get an instance and set the colnames...
	size = sti.getPropertyVector().size();
	pv = sti.getPropertyVector();
	
	const int instances =  querier->numElemInstances();
	colnames    = PROTECT(allocVector(VECSXP, size));
	result      = PROTECT(allocVector(VECSXP, size));
	each_column = new SEXP[size];

	for(int i = 0; i < size; i++)
	{
		switch(pv[i].attr_.rep_.type_)
		{
			case TeREAL:
				each_column[i] = PROTECT(allocVector(REALSXP,instances));
				break;
			case TeINT:
				each_column[i] = PROTECT(allocVector(INTSXP,instances));
				break;
			default:
				each_column[i] = PROTECT(allocVector(STRSXP,instances));
				break;
		}
		SET_STRING_ELT( colnames, i, mkChar(pv[i].attr_.rep_.name_.c_str()) );
	}
	
	// ... then we set all the values.
	// The while is at the end because we already read the first value
	do
	{
		pv = sti.getPropertyVector();
		for(int i = 0; i < size; i++)
		{
			// check the type, not all times will be real
	        switch(pv[i].attr_.rep_.type_)
    	    {
        	    case TeREAL:
       				REAL(each_column[i])[lines] = atof(pv[i].value_.c_str());
					break;
	            case TeINT:
       				INTEGER(each_column[i])[lines] = atoi(pv[i].value_.c_str());
					break;
        	    default:
       				SET_STRING_ELT(each_column[i], lines, mkChar(pv[i].value_.c_str()));
					break;
	        }
		}
		sti.clear();
		lines++;
	} while( querier->fetchInstance(sti) );
	
	setAttrib(result, R_NamesSymbol, colnames);

    for(int i = 0; i < size; i++) SET_VECTOR_ELT(result, i, each_column[i]);
	
	UNPROTECT(2 + size); // data and colnames, plus the number of attributes
	
	delete[] each_column;
	return result;
}

SEXP GetLinesTeMultiGeometry(TeMultiGeometry& mg)
{
	TeLineSet lineset;
	mg.getGeometry(lineset);

	return GetLinesTeLineSet(lineset);
}

SEXP GetLinesTeLineSet(TeLineSet& lineset)
{
	SEXP result, colnames;
	SEXP* each_column;
	int count = 0, size = 0;

    // count the number of points in each polygon
    for(unsigned i = 0; i < lineset.size(); i++)
        size += lineset[i].size();

	PROTECT(result   = allocVector(VECSXP, 2));
	PROTECT(colnames = allocVector(STRSXP, 2));
	
	each_column = new SEXP[2];

	PROTECT(each_column[0] = allocVector(REALSXP,size));
	PROTECT(each_column[1] = allocVector(REALSXP,size));
	SET_STRING_ELT( colnames, 0, mkChar("x") );
	SET_STRING_ELT( colnames, 1, mkChar("y") );
	
	// set the values in result
	for(unsigned i = 0; i < lineset.size(); i++)
	{
        for(unsigned j = 0; j < lineset[i].size(); j++)
		{
			REAL(each_column[0])[count] = lineset[i][j].x();
			REAL(each_column[1])[count] = lineset[i][j].y();
			count++;
		}
	}
	SET_VECTOR_ELT(result, 0, each_column[0]);	
	SET_VECTOR_ELT(result, 1, each_column[1]);	

	setAttrib(result, R_NamesSymbol, colnames);
	delete[] each_column;

	UNPROTECT(4);

	return result;
}

SEXP GetPolygonsTeMultiGeometry(TeMultiGeometry& mg)
{
	TePolygonSet polset;
	mg.getGeometry(polset);
    
	return GetPolygonsTePolygonSet(polset);
}

// ppol
SEXP GetPolygonsTePolygonSetSP(TePolygonSet& tepolset)
{
	SEXP pols = R_do_new_object(POLYGONS_DEF);
	SEXP Polygons, coords, pol, PlotOrder;
	
	PROTECT( Polygons  = allocVector(VECSXP, tepolset.size()));
	PROTECT( PlotOrder = allocVector(INTSXP, tepolset.size()));

	for(unsigned i = 0; i < tepolset.size(); i++)
	{
		for(unsigned j = 0; j < tepolset[i].size(); j++)
		{
			pol = R_do_new_object(POLYGON_DEF);
			
			for(unsigned k = 0; k < tepolset[i][j].size(); k++)
			{

			}
			
			R_do_slot_assign(pol, mkChar("coords"), coords);
		}
	}

	R_do_slot_assign(pols, mkChar("Polygons"),  Polygons );
	R_do_slot_assign(pols, mkChar("plotOrder"), PlotOrder);
	
	return pols;
}


// llls
/*
SEXP GetPolygonsTePolygonSetSP(TePolygonSet& tepolset)
{
	SEXP pols = R_do_new_object(POLYGONS_DEF);
	SEXP Polygons, coords, pol, PlotOrder;
	
	PROTECT( Polygons  = allocVector(VECSXP, tepolset.size()));
	PROTECT( PlotOrder = allocVector(INTSXP, tepolset.size()));

	for(unsigned i = 0; i < tepolset.size(); i++)
	{
		for(unsigned j = 0; j < tepolset[i].size(); j++)
		{
			pol = R_do_new_object(POLYGON_DEF);
			
			for(unsigned k = 0; k < tepolset[i][j].size(); k++)
			{

			}
			
			R_do_slot_assign(pol, mkChar("coords"), coords);
		}
	}

	R_do_slot_assign(pols, mkChar("Polygons"),  Polygons );
	R_do_slot_assign(pols, mkChar("plotOrder"), PlotOrder);
	
	return pols;
}
*/

SEXP GetPolygonsTePolygonSet(TePolygonSet& polset)
{
	SEXP result, colnames;
	SEXP* each_column;
	int count = 0, size = 0;
		
	// count the number of points in each polygon
	for(unsigned i = 0; i < polset.size(); i++)
		for(unsigned j = 0; j < polset[i].size(); j++)
			size += polset[i][j].size();

	PROTECT(result   = allocVector(VECSXP, 2));
	PROTECT(colnames = allocVector(STRSXP, 2));
	
	each_column = new SEXP[2];

	PROTECT(each_column[0] = allocVector(REALSXP,size));
	PROTECT(each_column[1] = allocVector(REALSXP,size));
	SET_STRING_ELT( colnames, 0, mkChar("x") );
	SET_STRING_ELT( colnames, 1, mkChar("y") );
	
	// set the values in result
	for(unsigned i = 0; i < polset.size(); i++)
	{
        for(unsigned j = 0; j < polset[i].size(); j++)
            for(unsigned k = 0; k < polset[i][j].size(); k++)
			{
				REAL(each_column[0])[count] = polset[i][j][k].x();
				REAL(each_column[1])[count] = polset[i][j][k].y();
				count++;
			}
	}
	SET_VECTOR_ELT(result, 0, each_column[0]);	
	SET_VECTOR_ELT(result, 1, each_column[1]);	

	setAttrib(result, R_NamesSymbol, colnames);
	delete[] each_column;

	UNPROTECT(4);

	return result;
}

SEXP GetPointsTeMultiGeometry(TeMultiGeometry& mg)
{
	TePointSet ps;
	mg.getGeometry(ps);

    return GetPointsTePointSet(ps);
}

SEXP GetPointsTePointSet(TePointSet& ps)
{
	const int size = ps.size() ;
	SEXP result, colnames, *each_column;
	
	PROTECT(result   = allocVector(VECSXP, 2));
	PROTECT(colnames = allocVector(STRSXP, 2));
	
	each_column = new SEXP[2];

	PROTECT(each_column[0] = allocVector(REALSXP,size));
    PROTECT(each_column[1] = allocVector(REALSXP,size));
	SET_STRING_ELT(colnames, 0, mkChar("x"));
	SET_STRING_ELT(colnames, 1, mkChar("y"));

	for (int j = 0; j < size; j++)
	{
        REAL(each_column[0])[j] = ps[j].location().x();
        REAL(each_column[1])[j] = ps[j].location().y();
    }
    SET_VECTOR_ELT(result, 0, each_column[0]);
    SET_VECTOR_ELT(result, 1, each_column[1]);

    setAttrib(result, R_NamesSymbol, colnames);
    delete[] each_column;

	UNPROTECT(4);
	return result;
}

// ppp
/*
SEXP GetPointsTePointSetSP(TePointSet& ps)
{ 
	const int size = ps.size() ;
	SEXP result, colnames, dimnames, *each_column;
	
	PROTECT(result   = allocVector(VECSXP, 2));
	PROTECT(colnames = allocVector(STRSXP, 2));
	PROTECT(dimnames = allocVector(INTSXP, size));
	
	each_column = new SEXP[2];

	PROTECT(each_column[0] = allocVector(REALSXP,size));
    PROTECT(each_column[1] = allocVector(REALSXP,size));
	SET_STRING_ELT(colnames, 0, mkChar("x"));
	SET_STRING_ELT(colnames, 1, mkChar("y"));

	for (int j = 0; j < size; j++)
	{
        REAL(each_column[0])[j] = ps[j].location().x();
        REAL(each_column[1])[j] = ps[j].location().y();

		INTEGER(dimnames)[j] = j;
	}
    SET_VECTOR_ELT(result, 0, each_column[0]);
    SET_VECTOR_ELT(result, 1, each_column[1]);

    setAttrib(result, R_NamesSymbol, colnames);
    delete[] each_column;

	result = AsDataFrame(result, rownames);
	UNPROTECT(4);
	return result;

  
		int nroptos = ps.size() ;
  SEXP coords, rownames, colnames, dimnames;
  
  PROTECT(coords   = allocMatrix(REALSXP, 2, nroptos));
  PROTECT(rownames = allocVector(INTSXP, nroptos));
  PROTECT(dimnames = allocVector(VECSXP, 2));
  PROTECT(colnames = allocVector(STRSXP, 2));
    
  SET_STRING_ELT(colnames, 0, mkChar("x"));
  SET_STRING_ELT(colnames, 1, mkChar("y"));

  for (int j = 0; j < nroptos; j++)
  { 
    REAL(coords)[0 + 2 * j] = ps[j].location().x();
    REAL(coords)[1 + 2 * j] = ps[j].location().y();
  
    const char *id = ps[j].objectId().c_str();

    // creates a vector with values rownames
    INTEGER(rownames)[j] = atoi(id);
  }

  SET_VECTOR_ELT(dimnames, 0, colnames);
  SET_VECTOR_ELT(dimnames, 1, rownames);

  setAttrib(coords, R_DimNamesSymbol, dimnames);
  UNPROTECT(4);
  return coords;
}*/


SEXP CharToSEXP(char* vchar)
{
	SEXP value = allocVector(STRSXP, 1);         
    SET_STRING_ELT(value, 0, mkChar(vchar));
	return value;
}
	
int GetPos(SEXP names, string value)
{
//		printSEXP(names);
//		cout << "aq" << endl;
	for(int i = 0; i < LENGTH(names);i++)
	{
//		cout << i << endl;
//		cout << CHAR(STRING_ELT(names, i)) << endl;
//		cout << "iiiii" << endl;
		
		if(value == CHAR(STRING_ELT(names, i)))
		{
//				cout << "return" << endl;
			return i;
		}
//		cout << i << endl;
	}
	return -1;
}

string GetStringItemPos(SEXP value, int pos)
{
	stringstream s;
	switch( TYPEOF(value) )
   	{
		case REALSXP: s << REAL(value)[pos];            break;
       	case INTSXP:  s << INTEGER(value)[pos];         break;
       	case STRSXP:  s << CHAR(STRING_ELT(value,pos)); break;
        default: cerr << "IDs type unrecognized" << endl;
	}
	return s.str().c_str();
}

TePointSet GetTePointSet(SEXP sps)
{
	TePointSet ps;
	
	SEXP data = R_do_slot( sps, CharToSEXP("data")   );
	SEXP geom = R_do_slot( sps, CharToSEXP("coords") );

	data = R_do_slot( data, CharToSEXP("att"));
	SEXP ids  = GetListElement(data, "ID");

	const int size = LENGTH(ids);
	
	for(int i = 0; i < size; i++)
	{
		TePoint point(REAL(geom)[i], REAL(geom)[i+size] );
		point.objectId( GetStringItemPos(ids, i) );
		ps.add(point);
	}
	
	return ps;
}

// old class aRTgeometry. Deprecated
TePointSet GetTePointSetSP(SEXP sps)
{
	TePointSet ps;

	SEXP ids  = GetListElement(sps, aRTstrId);
	SEXP geom = GetListElement(sps, aRTstrPoint);
	
	const int size = LENGTH(ids);

	for(int i = 0; i < size; i++)
	{
		if(VECTOR_ELT(geom,i) != R_NilValue)
		{
			SEXP arx = GetListElement(VECTOR_ELT(geom,i), "x");
			SEXP ary = GetListElement(VECTOR_ELT(geom,i), "y");

			const int internal_size = LENGTH(arx);

			for(int j = 0; j < internal_size; j++) // one id can have more than one point
			{
				TePoint point(REAL(arx)[j], REAL(ary)[j]);
				point.objectId( CHAR(STRING_ELT(ids, i)) );
				ps.add(point);
			}
		}
	}
	return ps;
}

TePolygonSet GetTePolygonSet(SEXP sps)
{
    TePolygonSet ps;

    SEXP geom = R_do_slot( sps, CharToSEXP("polygons") );

	const int size = LENGTH(geom);

    for(int i = 0; i < size; i++)
    {
		SEXP srings = R_do_slot( VECTOR_ELT(geom, i), CharToSEXP("Polygons") );
		SEXP ID     = R_do_slot( VECTOR_ELT(geom, i), CharToSEXP("ID") );

       	const int rings_size = LENGTH(srings);
		TePolygon pol;
		for(int j = 0; j < rings_size; j++)
		{
			SEXP sring = R_do_slot( VECTOR_ELT(srings, j), CharToSEXP("coords") );
			SEXP hole  = R_do_slot( VECTOR_ELT(srings, j), CharToSEXP("hole") );

			bool hooooooooooooooooole = LOGICAL(hole)[0];
			const int ring_size = LENGTH(sring)/2;
			TeLine2D line;
			for(int k = 0; k < ring_size; k++)
			{
				TeCoord2D coord(REAL(sring)[k], REAL(sring)[k+ring_size] );
				line.add(coord);
			}

			if(!hooooooooooooooooole && !pol.empty())
			{
				pol.objectId( CHAR(STRING_ELT(ID, 0)) );
				ps.add(pol);
				pol = TePolygon();
			}

            TeLinearRing lr(line);
            pol.add(lr);

        }
		if(!pol.empty())
		{
			pol.objectId( CHAR(STRING_ELT(ID, 0)) );
			ps.add(pol);
		}
    }
	return ps;
}

// old class aRTgeometry. Deprecated
TePolygonSet GetTePolygonSetSP(SEXP sps)
{
    TePolygonSet ps;

	SEXP ids  = GetListElement(sps, aRTstrId);
    SEXP geom = GetListElement(sps, aRTstrPolygon);

    const int size = LENGTH(ids);

    for(int i = 0; i < size; i++)
    {
		if(VECTOR_ELT(geom,i) != R_NilValue)
		{
        	SEXP arx = GetListElement(VECTOR_ELT(geom,i), "x");
        	SEXP ary = GetListElement(VECTOR_ELT(geom,i), "y");

        	const int internal_size = LENGTH(arx);

			//TePolygon pol;
			for(int j = 0; j < internal_size; )
			{
				TeLine2D line;
				TePolygon pol;

    	        TeCoord2D coord(REAL(arx)[j], REAL(ary)[j]);
				line.add(coord);
				j++;
	
				do
				{
	            	TeCoord2D coord(REAL(arx)[j], REAL(ary)[j]);
					line.add(coord);
					j++;
				} while (line.last() != line.first());
			
				TeLinearRing lr(line);
				//TePolygon pol;
				pol.objectId( CHAR(STRING_ELT(ids, i)) );
	
				pol.add(lr);
				ps.add(pol);
				//line.clear(); // <= ESSA MERDA!
	        }
				//ps.add(pol);
		}
    }
	return ps;
}

TeLineSet GetTeLineSet(SEXP sls)
{
//	cout << "aqui" << endl;
	TeLineSet ls;

    SEXP geom = R_do_slot( sls, CharToSEXP("lines") );

	const int size = LENGTH(geom);
//	cout << "size: " << size << endl;

    for(int i = 0; i < size; i++)
    {
		SEXP slines = R_do_slot( VECTOR_ELT(geom, i), CharToSEXP("Lines") );
		SEXP ID     = R_do_slot( VECTOR_ELT(geom, i), CharToSEXP("ID") );

//		printSEXP(ID);

       	const int lines_size = LENGTH(slines);
//		cout << "rings_size " << lines_size << endl;
//		TeLine2D pol;
		for(int j = 0; j < lines_size; j++)
		{
			SEXP sline = R_do_slot( VECTOR_ELT(slines, j), CharToSEXP("coords") );

//			printSEXP(sline);
			const int line_size = LENGTH(sline)/2;
//			cout << "ring_size " << line_size << endl;
			TeLine2D line;
			for(int k = 0; k < line_size; k++)
			{
				TeCoord2D coord(REAL(sline)[k], REAL(sline)[k+line_size] );
				line.add(coord);
			}

			line.objectId( CHAR(STRING_ELT(ID, 0)) );
//			cout << "adicionando linha de tamanho " << line.size() << endl;
			ls.add(line);
        }
//		pol.objectId( CHAR(STRING_ELT(ID, 0)) );
//		ls.add(pol);
    }
	return ls;
}

// old class aRTgeometry. Deprecated
TeLineSet GetTeLineSetSP(SEXP sls)
{
    TeLineSet ls;
	SEXP ids  = GetListElement(sls, aRTstrId);
    SEXP geom = GetListElement(sls, aRTstrLine);

    const int size = LENGTH(ids);

    for(int i = 0; i < size; i++)
    {
		if(VECTOR_ELT(geom,i) != R_NilValue)
		{
        	SEXP arx = GetListElement(VECTOR_ELT(geom,i), "x");
        	SEXP ary = GetListElement(VECTOR_ELT(geom,i), "y");

        	const int internal_size = LENGTH(arx);
			TeLine2D line;

			// TODO: more than one line per object
			for(int j = 0; j < internal_size; j++)
			{
        	    TeCoord2D coord(REAL(arx)[j], REAL(ary)[j]);
				line.add(coord);
			}

			line.objectId( CHAR(STRING_ELT(ids, i)) );
			ls.add(line);
		}
    }
	return ls;
}

SEXP GetCellsTeMultiGeometry(TeMultiGeometry&)
{
	return R_NilValue;
}

enum Geometries
{
	GeomPoints,
	GeomPolygons,
	GeomLines,
	GeomSize
};

const char* GeometryNames[] =
{
    aRTstrPoint,
    aRTstrPolygon,
	aRTstrLine
};

SEXP GetLayerGeometry(TeLayer* layer)
{
    TeInitQuerierStrategies(); // never forget it!
    TeQuerierParams params(true, false); // only geometry, no attributes
    params.setParams(layer);
    TeQuerier* querier = new TeQuerier(params);
	
	SEXP result = GetQuerierGeometry(querier);
	delete querier;
	return result;
}

SEXP GetThemeGeometry(TeTheme* theme)
{
    TeInitQuerierStrategies(); // never forget it!
    
    TeQuerierParams params(true, false); // only geometry, no attributes
    params.setParams(theme);
    TeQuerier* querier = new TeQuerier(params);
	
	SEXP result = GetQuerierGeometry(querier);
	delete querier;
	return result;
}

SEXP GetQuerierGeometry(TeQuerier* querier)
{	
	SEXP colnames, result;
	TeSTInstance sti;
	TeMultiGeometry mg;
	int lines = 0;
	int geometries = 0;
	SEXP vgeometries[GeomSize];
	SEXP ids;
	
	for(int i = 0; i < GeomSize; i++)
		vgeometries[i] = R_NilValue;
	
	if( !querier->loadInstances() )
	{
		cerr << "SEXPutils: Can't load instances" << endl;
		return R_NilValue;
	}
	if( !querier->fetchInstance(sti) )
	{
		cerr << "SEXPutils: Can't fetch instances" << endl;
		return R_NilValue;
	}
	
	const int instances =  querier->numElemInstances();
//	cout << "instances:" << instances << endl;
	
	#define IF_NIL_ALLOC_AND_SET(object)           \
	if(object == R_NilValue)                       \
	{                                              \
		object = allocVector(VECSXP, instances);   \
		PROTECT(object);                           \
	    geometries ++;                             \
		for(int i = 0; i < instances; i++)         \
	        SET_VECTOR_ELT(object, i, R_NilValue); \
	}
	
    PROTECT(ids = allocVector(STRSXP, instances));

	do{
//		cout << "one instance" << endl;
		mg.clear();
		mg = sti.geometries();
   
		SET_STRING_ELT(ids, lines, mkChar(sti.objectId().c_str()));

		if( mg.hasPolygons() )
		{   // an instance can have more then one polygon. In this case,
			// each one is connected with the previous and the next by
			// "invisible" edges (except the first and the last, that have only one)
			// the first point of a polygon is repeated at its end
			IF_NIL_ALLOC_AND_SET(vgeometries[GeomPolygons]);
			SET_VECTOR_ELT(vgeometries[GeomPolygons], lines, GetPolygonsTeMultiGeometry(mg));
		}
		
		if( mg.hasPoints() )
		{
			IF_NIL_ALLOC_AND_SET(vgeometries[GeomPoints]);
			SET_VECTOR_ELT(vgeometries[GeomPoints], lines, GetPointsTeMultiGeometry(mg));
		}

		if( mg.hasLines() )
		{
			IF_NIL_ALLOC_AND_SET(vgeometries[GeomLines]);
			SET_VECTOR_ELT(vgeometries[GeomLines], lines, GetLinesTeMultiGeometry(mg));
		}
		if( mg.hasCells() ) cout << "cells not supported yet" << endl;
		if( mg.empty() )    cout << "found an empty geometry: " << lines << endl;
		mg.clear();
		sti.clear();
		lines++;
	} while( querier->fetchInstance(sti) );

	if(geometries)
	{
	    PROTECT(colnames = allocVector(STRSXP, geometries+1));
	    PROTECT(result   = allocVector(VECSXP, geometries+1));
		
		int j = 0;
		for(int i = 0; i != GeomSize; i++)
		{
			if(vgeometries[i] != R_NilValue)
			{
			    SET_STRING_ELT(colnames, j, mkChar(GeometryNames[i]));
				SET_VECTOR_ELT(result,   j, vgeometries[i]);
				j++;
			}
		}

		SET_STRING_ELT(colnames, j, mkChar(aRTstrId));
		SET_VECTOR_ELT(result,   j, ids);

		setAttrib(result, R_NamesSymbol, colnames);

		UNPROTECT(geometries + 3); // + colnames + result + ids
		return result;
	}
	cerr << "SEXPutils: Error at getgeom" << endl;
	UNPROTECT(geometries + 1);
	return R_NilValue;
}

/*
void printAttrib(SEXP e)
{
	SEXP rn=Rf_GetRowNames(e);
	if(rn != R_NilValue)
	{
		printf("Row names:\n");
		printSEXP(rn);
	}

	SEXP cn=getAttrib(e, R_NamesSymbol);
	{
		printf("Col names:\n");
		printSEXP(cn);
	}
}*/

void printSEXP(SEXP e)
{
    int i = 0;

    switch( TYPEOF(e) )
    {
        case NILSXP:
            printf("NULL value\n");
            return;

        case LANGSXP:
            printf("language construct\n");
            return;

        case REALSXP:
            if (LENGTH(e) > 1)
            {
                printf("Vector of real variables: ");
                while( i < LENGTH(e) )
                {
                    printf("%f",REAL(e)[i]);
                    if ( !(REAL(e)[i] < 0) && !(REAL(e)[i] >= 0) ) printf("__NAN__");
                    if (i < LENGTH(e) - 1) printf(", ");
                    if (dumpLimit && i > dumpLimit) { printf("..."); break; }
                    i++;
                }
                putchar('\n');
            }
            else printf( "Real variable %f\n", *REAL(e) );
            return;
        case EXPRSXP:
            printf( "Vector of %d expressions:\n", LENGTH(e) );
            while( i < LENGTH(e) )
            {
                if (dumpLimit && i > dumpLimit){ printf("...");  break; }
                printSEXP( VECTOR_ELT(e,i) );
                i++;
            }
            return;

         case INTSXP:
            printf( "Vector of %d integers:\n", LENGTH(e) );

            while( i < LENGTH(e) )
            {
                if (dumpLimit && i > dumpLimit) { printf("..."); break; }
                printf("%d",INTEGER(e)[i]);
                if (i < LENGTH(e) - 1) printf(", ");
                i++;
            };
            putchar('\n');
            return;

        case VECSXP:
            printf( "Vector of %d fields:\n", LENGTH(e) );
            while( i < LENGTH(e) )
            {
                if (dumpLimit && i>dumpLimit) { printf("..."); break; };
                printSEXP(VECTOR_ELT(e,i));
                i++;
            }
            return;

        case STRSXP:
            printf( "String vector of length %d:\n", LENGTH(e) );
            while( i < LENGTH(e) )
            {
                if (dumpLimit && i > dumpLimit) { printf("..."); break; };
                printSEXP(VECTOR_ELT(e,i)); i++;
            }
            return;
	   case CHARSXP:
            printf( "scalar string: \"%s\"\n",(char*) STRING_PTR(e) );
            return;

        case SYMSXP:
            printf( "Symbol, name: "); printSEXP(PRINTNAME(e) );
            return;

        default:
            printf( "Unknown type: %d\n", TYPEOF(e) );
    }
};

string AttrDataType(TeAttrDataType value)
{
    switch (value)
    {
        case TeSTRING:    return "string";
        case TeREAL:      return "real";
        case TeINT:       return "integer";
        case TeDATETIME:  return "date";
        case TeBLOB:      return "blob";
        case TeOBJECT:    return "object";
        case TeCHARACTER: return "character";
        case TeUNKNOWN:
		default:          return "<unknown>";
    }
}

string AttrTableType(TeAttrTableType value)
{
    switch(value)
    {
        case TeAttrStatic:       return "static";
        case TeAttrMedia:        return "media";
        case TeAttrExternal:     return "external";
        case TeAttrEvent:        return "event";
        case TeFixedGeomDynAttr: return "dynattr";
        case TeDynGeomDynAttr:   return "alldyn";
        case TeGeomAttrLinkTime: return "dyngeom";
        case TeGeocodingData:
        default:                 return "<unrecognized type>";
    }
}

TeChronon GetTeChronon(string value)
{
	switch(value[0])
	{
		case 'y': return TeYEAR;
		case 's': return TeSECOND;
		case 'c': return TeSEASON;
		case 'f': return TeSECONDOFMINUTE;
		case 'v': return TeWEEKOFYEAR;
		case 'm':
			switch(value[2])
			{
				case 'i': return value.size() == 6? TeMINUTE : TeMINUTEOFHOUR;
				case 'o': return value.size() == 5? TeMONTH  : TeMONTHOFYEAR;
			}
		case 'h': return value.size() == 4? TeHOUR : TeHOUROFDAY;
		case 'd':
			if(value.size() == 3) return TeDAY;
			switch(value[5])
			{
				case 'w': return TeDAYOFWEEK;
        		case 'm': return TeDAYOFMONTH;
				case 'y': return TeDAYOFYEAR;
			}
		case 'n':
	    default:  return TeNOCHRONON;
	}
}

TeStatisticType GetTeStatisticType(string value)
{
	switch(value[0])
	{
        case 'a': return TeAMPLITUDE;
		case 'c': return TeCOUNT;
		case 'k': return value[1] == 'e'? TeKERNEL : TeKURTOSIS;
		case 's':
			switch(value[1])
			{
				case 'u': return TeSUM;
				case 't': return TeSTANDARDDEVIATION;
				case 'k': return TeSKEWNESS;
			}
		case 'm':
			switch(value[1])
			{
				case 'a': return TeMAXVALUE;
				case 'i': return TeMINVALUE;
				case 'e': return value[2] == 'a'? TeMEAN : TeMEDIAN;
				case 'o': return TeMODE;
			}
		case 'v':
			switch(value[4])
			{
				case 'd': return TeVALIDCOUNT;
				case 'o': return TeVARCOEFF;
				case 'a': return TeVARIANCE;
			}
		case 'n':
		default:  return TeNOSTATISTIC;
	}
}

}

}

